
#include "Quad.fxh"

float4x4 tW : World;
float4x4 tWIT : WorldInverseTranspose;
float4x4 tWVP : WorldViewProjection;
float4x4 tVI : ViewInverse;
float4x4 tWV : WorldView;

/* data from application vertex buffer */
struct appdata {
    float3 Pos    : POSITION;
    float4 UV        : TEXCOORD0;
    float4 Normal    : NORMAL;
    float4 Tangent    : TANGENT0;
    float4 Binormal    : BINORMAL0;
};


/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float3 WorldNormal	: TEXCOORD0;
    float3 WorldEyeVec	: TEXCOORD1;
    float3 EyePos	: TEXCOORD2;
// float4 Diff		: COLOR0;
};

vertexOutput simpleVS(appdata IN)
{
    vertexOutput OUT = (vertexOutput)0;
    float4 Po = float4(IN.Pos,1.0);
    OUT.HPosition = mul(Po,tWVP);
    float4 Nn = normalize(IN.Normal);
    OUT.WorldNormal = mul(tWIT,Nn).xyz;
    float4 Pw = mul(tW,Po);
    OUT.EyePos = mul(Po,tWV).xyz;
    OUT.WorldEyeVec = normalize(tVI[3].xyz - Pw.xyz);
    return OUT;
}

///pixel shaders ////

QUAD_REAL4 vecColorN(QUAD_REAL3 V) {
    QUAD_REAL3 Nc = 0.5 * (normalize(V.xyz) + ((1.0).xxx));
    return QUAD_REAL4(Nc,1);
}

QUAD_REAL4 normPS(vertexOutput IN): COLOR
{

return vecColorN(IN.WorldNormal);

}

////////////////////////////////////////////////////////////
/////////////////////////////////////// techniques /////////
////////////////////////////////////////////////////////////

technique Main
{
	pass Norms
	{
		VertexShader = compile vs_3_0 simpleVS();
 	PixelShader = compile ps_3_0 normPS();
	}
    }


////////////// eof ///
