//*****************************************************************/

#include "Quad.fxh"

// transform object vertices to view space and project them in perspective:
float4x4 tWVP : WorldViewProjection;

//////////

float NPixels <
    string UIName = "Edge Pixels Steps";
    string UIWidget = "slider";
    float UIMin = 1.0f;
    float UIMax = 5.0f;
    float UIStep = 0.5f;
> = 1.5f;

float Threshhold <
    string UIName = "Edge Threshhold";
    string UIWidget = "slider";
    float UIMin = 0.001f;
    float UIMax = 0.1f;
    float UIStep = 0.001f;
> = 0.2;

float ThreshholdD <
    string UIName = "Depth Threshhold";
    string UIWidget = "slider";
    float UIMin = 0.0001f;
    float UIMax = 0.1f;
    float UIStep = 0.0001f;
> = 0.2;

///////////////////////////////////////////////////////////
///////////////////////////// Render-to-Texture Data //////
///////////////////////////////////////////////////////////

DECLARE_QUAD_TEX(gNormTexture,gNormSampler,"X8R8G8B8")
DECLARE_QUAD_TEX(gDeepTexture,gDeepSampler,"X8R8G8B8")
DECLARE_QUAD_DEPTH_BUFFER(gDepthBuffer, "D24S8")


// static data //////////////////////////////////

static float EdgeT2 = (Threshhold * Threshhold);
static float DeepT2 = (ThreshholdD * ThreshholdD);


//////////////////////////// geometric data ///////

/////// edge detection //////////////

// packing macros
#define UV00 TC00.xy
#define UV01 TC00.zw
#define UV02 TC01.xy
#define UV10 TC01.zw
#define UV11 TC02.xy
#define UV12 TC02.zw
#define UV20 TC03.xy
#define UV21 TC03.zw
#define UV22 TC04.xy

struct EdgeVertexOutput
{
    QUAD_REAL4 Position	: POSITION;
    QUAD_REAL4 TC00	: TEXCOORD0; // packed
    QUAD_REAL4 TC01	: TEXCOORD1; // packed
    QUAD_REAL4 TC02	: TEXCOORD2; // packed
    QUAD_REAL4 TC03	: TEXCOORD3; // packed
    QUAD_REAL2 TC04	: TEXCOORD4; // not packed
};


EdgeVertexOutput edgeVS(
		QUAD_REAL3 Position : POSITION,
		QUAD_REAL3 TexCoord : TEXCOORD0,
		uniform float2 ScreenSize,
		uniform float2 TexelCornerOffset,
		uniform float NPixels
) {
    EdgeVertexOutput OUT = (EdgeVertexOutput)0;
    OUT.Position = QUAD_REAL4(Position, 1);
    QUAD_REAL2 ctr = QUAD_REAL2(TexCoord.xy+TexelCornerOffset);
    QUAD_REAL2 ox = QUAD_REAL2(NPixels/ScreenSize.x,0.0);
    QUAD_REAL2 oy = QUAD_REAL2(0.0,NPixels/ScreenSize.y);
    OUT.UV00 = ctr - ox - oy;
    OUT.UV01 = ctr - oy;
    OUT.UV02 = ctr + ox - oy;
    OUT.UV10 = ctr - ox;
    OUT.UV11 = ctr;
    OUT.UV12 = ctr + ox;
    OUT.UV20 = ctr - ox + oy;
    OUT.UV21 = ctr + oy;
    OUT.UV22 = ctr + ox + oy;
    
    return OUT;
}

  //////////////////////
 // pixel shader //////
//////////////////////

//
// Edge detect against gray
//
// Note for THIS conversion, we treat R G and B *equally* -- a better
//	value for the dot product for a regular color image might be something
//	perceptual like (.2,.7,.1) -- but in this case we are filtering 3D normals
//  so (0.333).xxx is good
//
QUAD_REAL getGray(QUAD_REAL4 c)
{
    return(dot(c.rgb,((0.33333).xxx)));
}

QUAD_REAL edgeDetectGray(EdgeVertexOutput IN,
						 uniform sampler2D ColorMap,
						 uniform QUAD_REAL T2) {
	QUAD_REAL4 CC;
	CC = tex2D(ColorMap,IN.UV00); QUAD_REAL g00 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV01); QUAD_REAL g01 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV02); QUAD_REAL g02 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV10); QUAD_REAL g10 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV12); QUAD_REAL g12 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV20); QUAD_REAL g20 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV21); QUAD_REAL g21 = getGray(CC);
	CC = tex2D(ColorMap,IN.UV22); QUAD_REAL g22 = getGray(CC);
	QUAD_REAL sx = 0;
	sx -= g00;
	sx -= g01 * 2;
	sx -= g02;
	sx += g20;
	sx += g21 * 2;
	sx += g22;
	QUAD_REAL sy = 0;
	sy -= g00;
	sy += g02;
	sy -= g10 * 2;
	sy += g12 * 2;
	sy -= g20;
	sy += g22;
	QUAD_REAL dist = (sx*sx+sy*sy);
	QUAD_REAL result = 0;
	if (dist>T2) { result = 1; }
	return result;
}

//
// Only the red channel since the image is grayscale already
//
QUAD_REAL edgeDetectR(EdgeVertexOutput IN,
						 uniform sampler2D ColorMap,
						 uniform QUAD_REAL T2) {
	QUAD_REAL4 CC;
	QUAD_REAL g00 = tex2D(ColorMap,IN.UV00).x;
	QUAD_REAL g01 = tex2D(ColorMap,IN.UV01).x;
	QUAD_REAL g02 = tex2D(ColorMap,IN.UV02).x;
	QUAD_REAL g10 = tex2D(ColorMap,IN.UV10).x;
	QUAD_REAL g12 = tex2D(ColorMap,IN.UV12).x;
	QUAD_REAL g20 = tex2D(ColorMap,IN.UV20).x;
	QUAD_REAL g21 = tex2D(ColorMap,IN.UV21).x;
	QUAD_REAL g22 = tex2D(ColorMap,IN.UV22).x;
	QUAD_REAL sx = 0;
	sx -= g00;
	sx -= g01 * 2;
	sx -= g02;
	sx += g20;
	sx += g21 * 2;
	sx += g22;
	QUAD_REAL sy = 0;
	sy -= g00;
	sy += g02;
	sy -= g10 * 2;
	sy += g12 * 2;
	sy -= g20;
	sy += g22;
	QUAD_REAL dist = (sx*sx+sy*sy);
	QUAD_REAL result = 0;
	if (dist>T2) { result = 1; }
	return result;
}

///////

QUAD_REAL4 edgeDetectPS(EdgeVertexOutput IN,
		    uniform sampler2D NormSampler,
		    uniform sampler2D DeepSampler,
		    uniform float EdgeT2,
		    uniform float DeepT2
) : COLOR {
    QUAD_REAL n = edgeDetectGray(IN,NormSampler,EdgeT2);
    QUAD_REAL d = edgeDetectR(IN,DeepSampler,DeepT2);
    QUAD_REAL ink = 1 - (n*d);
    return ink.xxxx;
}

QUAD_REAL4 normEdgePS(EdgeVertexOutput IN,
		    uniform sampler2D NormSampler,
		    uniform float EdgeT2
) : COLOR {
    QUAD_REAL d = 1.0 - edgeDetectGray(IN,NormSampler,EdgeT2);
    float4 dc = tex2D(NormSampler,IN.UV11).x;
    return float4(dc.rgb,1.0);
    return d.xxxx;
}

QUAD_REAL4 deepEdgePS(EdgeVertexOutput IN,
		    uniform sampler2D DeepSampler,
		    uniform float DeepT2
) : COLOR {
    QUAD_REAL d = 1.0 - edgeDetectR(IN,DeepSampler,DeepT2);
    float dv = tex2D(DeepSampler,IN.UV11).x;
    return float4(dv.xxx,1.0);
    return d.xxxx;
}

////////////////////////////////////////////////////////////
/////////////////////////////////////// techniques /////////
////////////////////////////////////////////////////////////

technique Main
{
    pass ImageProc
     {
		VertexShader = compile vs_3_0 edgeVS(QuadScreenSize,QuadTexelOffsets,NPixels);
		PixelShader = compile ps_3_0 edgeDetectPS(gNormSampler,
							    gDeepSampler,
							    EdgeT2,DeepT2);
    }
}
